@@ -80,6 +80,10 @@ constantTimeCondition(unsigned int c,
 static unsigned int
 rsa_modulusLen(SECItem *modulus)
 {
+    if (modulus->len == 0) {
+        return 0;
+    }
+
     unsigned char byteZero = modulus->data[0];
     unsigned int modLen = modulus->len - !byteZero;
     return modLen;
@@ -88,9 +92,17 @@ rsa_modulusLen(SECItem *modulus)
 static unsigned int
 rsa_modulusBits(SECItem *modulus)
 {
+    if (modulus->len == 0) {
+        return 0;
+    }
+
     unsigned char byteZero = modulus->data[0];
     unsigned int numBits = (modulus->len - 1) * 8;
 
+    if (byteZero == 0 && modulus->len == 1) {
+        return 0;
+    }
+
     if (byteZero == 0) {
         numBits -= 8;
         byteZero = modulus->data[1];
@@ -147,7 +159,7 @@ rsa_FormatOneBlock(unsigned modulusLen,
             padLen = modulusLen - data->len - 3;
             PORT_Assert(padLen >= RSA_BLOCK_MIN_PAD_LEN);
             if (padLen < RSA_BLOCK_MIN_PAD_LEN) {
-                PORT_Free(block);
+                PORT_ZFree(block, modulusLen);
                 return NULL;
             }
             PORT_Memset(bp, RSA_BLOCK_PRIVATE_PAD_OCTET, padLen);
@@ -176,7 +188,7 @@ rsa_FormatOneBlock(unsigned modulusLen,
             padLen = modulusLen - (data->len + 3);
             PORT_Assert(padLen >= RSA_BLOCK_MIN_PAD_LEN);
             if (padLen < RSA_BLOCK_MIN_PAD_LEN) {
-                PORT_Free(block);
+                PORT_ZFree(block, modulusLen);
                 return NULL;
             }
             j = modulusLen - 2;
@@ -205,7 +217,7 @@ rsa_FormatOneBlock(unsigned modulusLen,
                 }
             }
             if (rv != SECSuccess) {
-                PORT_Free(block);
+                PORT_ZFree(block, modulusLen);
                 PORT_SetError(SEC_ERROR_LIBRARY_FAILURE);
                 return NULL;
             }
@@ -216,7 +228,7 @@ rsa_FormatOneBlock(unsigned modulusLen,
 
         default:
             PORT_Assert(0);
-            PORT_Free(block);
+            PORT_ZFree(block, modulusLen);
             return NULL;
     }
 
